#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
#include <sstream>
#include <curl/curl.h>


using namespace std;

struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data(istream& stream) {
    Input in;
    size_t number_count;
    stream >> number_count;
    in.numbers.resize(number_count);
    for (size_t i = 0; i < number_count; i++) {
        stream >> in.numbers[i];
    }
    stream >> in.bin_count;
    return in;
}

size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}


Input download(const string& address) {
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if (curl) {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        if (res != CURLE_OK) {
            cout << curl_easy_strerror(res);
            exit(1);
        }
        auto data = curl_version_info(CURLVERSION_NOW)->protocols;
            for (auto protocol = data; *protocol; ++protocol) {
            cerr << *protocol << endl;
        }
        curl_easy_cleanup(curl);
    }
        return input_data(buffer);
}

int main(int argc, char* argv[]) {
    Input in;
    if (argc > 1) {
        in = download(argv[1]);
    } else {
        in = input_data(cin);
    }
    auto bins = make_histogram(in.numbers, in.bin_count);
    show_histogram_svg(bins);
    cerr << "cURL version " << curl_version_info(CURLVERSION_NOW)->version << endl;
    auto ssl_ver = curl_version_info(CURLVERSION_NOW)->ssl_version;
    cerr << "SSL version " << ssl_ver << "\n";
    return 0;
}
